/*
 * Gaim Everything2 Plugin
 *
 * Copyright (C) John V. Baboval 
 * Copyright (C) 2004, Frederick Heckel (rewritten for new versions of gaim)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "internal.h"
#include "debug.h"
#include "plugin.h"
#include "gtkutils.h"
#include "util.h"
#include <string.h>
#include <stdbool.h>

#define LINK "<A HREF=\"http://www.everything2.com/index.pl?node="
#define ENDLINK "</A>"
#define LINKLEN 55
#define PLUG_VER "0.75"

// much of this code is from the old plugin by John V. Baboval
static gboolean parse_e2(GaimAccount *account, GaimConversation *conv, char **message, void *data){

  char *msg=*message;
  int len,i,j,start_ind,end_ind,hlen,size;
  char *front,*middle,*end,*href;
  char *temp,*temp2,*new,*place;
  bool in_link=false;
  
  switch(gaim_account_get_protocol(account)){
  case GAIM_PROTO_IRC:
    // this'll just piss people off over irc, since most irc clients
    // don't know about html. maybe make this configurable later.
    gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug", "Not parsing IRC message.\n");
    return FALSE;
    break;
  default:
    break;
  }

  gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug", "Starting to parse the message\n");

  len=strlen(msg);

  for(i=0;i<len;i++){
    if(msg[i]=='['){
      // this lets us escape a bracket
      if(i>0&&msg[i-1]=='\\'){
	//and this gives us one special case
	if(msg[i-2]=='\\'){
	  hlen=0;
	  front=NULL;
	  middle=NULL;
	  end=NULL;
	  href=NULL;
	  temp=NULL;
	  temp2=NULL;
	  place=NULL;
	  start_ind=i-1;
	  in_link=true;
	}
	for(j=i-1;j<strlen(msg);j++){
	  msg[j]=msg[j+1];
	}	
	
      }else{
	hlen=0;
	front=NULL;
	middle=NULL;
	end=NULL;
	href=NULL;
	temp=NULL;
	temp2=NULL;
	place=NULL;
	start_ind=i;
	in_link=true;
      }
    }else if(msg[i]==']'&&in_link){
      end_ind=i;
      front=msg;
      middle=msg;
      strsep(&middle,"[");
      end=middle;
      strsep(&end,"]");
      for(j=0;j<strlen(middle);j++){
	if(middle[j]==' '){
	  hlen+=3;
	}else{
	  hlen++;
	}
      }
      href=(char *)malloc(sizeof(char)*hlen+1);
      temp=(char *)malloc(strlen(middle)+1);
      temp2=place=temp;

      strncpy(temp,middle,strlen(middle)+1);
      strsep(&temp2," ");
      strncpy(href,temp,sizeof(char)*hlen+1);

      while(temp2){
	strncat(href,"%20",sizeof(char)*hlen+1-strlen(href));
	temp=temp2;
	strsep(&temp2," ");
	strncat(href,temp,sizeof(char)*hlen+1-strlen(href));
      }
      strncat(href,"\0",sizeof(char)*hlen+1-strlen(href));
      size=strlen(front)+strlen(middle)+strlen(href)+strlen(end)+LINKLEN+5;

      new=(char *)malloc(sizeof(char)*size);
      gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug","Constructing message...\n");  

      snprintf(new,size*sizeof(char),"%s%s%s\">%s</a>%s",front,LINK,href,middle,end);

      len=strlen(new);
      i=(len-strlen(end))/sizeof(char);
      msg=new;

      gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug","Freeing memory...\n");  
      free(front);
      free(href);
      free(place);
      new=NULL;
    }

  }

  *message=msg;
  gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug", "Done...\n");  

  return FALSE;
}

static gboolean
plugin_load(GaimPlugin *plugin)
{
	gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug", "E2New-Plugin loaded.\n");

	void *conv_handle = gaim_conversations_get_handle();

	gaim_signal_connect(conv_handle, "displaying-im-msg",
						plugin, GAIM_CALLBACK(parse_e2), NULL);
	gaim_signal_connect(conv_handle, "displaying-chat-msg",
						plugin, GAIM_CALLBACK(parse_e2), NULL);


	return TRUE;
}

static gboolean
plugin_unload(GaimPlugin *plugin)
{
	gaim_debug(GAIM_DEBUG_INFO, "E2New-Plug", "E2New-Plugin unloaded.\n");

	return TRUE;
}

static GaimPluginInfo info =
{
	2,                                                /**< api_version    */
	GAIM_PLUGIN_STANDARD,                             /**< type           */
    NULL,                                             /**< ui_requirement */
	0,                                                /**< flags          */
	NULL,                                             /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                            /**< priority       */

	NULL,                                             /**< id             */
	N_("E2New-Plugin"),                              /**< name           */
	PLUG_VER,                                          /**< version        */
	                                                  /**  summary        */
	N_("New Everything2 Plugin that works with GAIM .7x"),
	                                                  /**  description    */
	N_("Replaces [bracketed] text with links to everything2. Original plugin by John V. Baboval."),
	"Frederick Heckel <fwph@sccs.swarthmore.edu>",        /**< author         */
	"http://www.sccs.swarthmore.edu/~fwph/code.html",          /**< homepage       */

	plugin_load,                                      /**< load           */
	plugin_unload,                                    /**< unload         */
	NULL,                                             /**< destroy        */

	NULL,                                             /**< ui_info        */
	NULL                                              /**< extra_info     */
};

static void
init_plugin(GaimPlugin *plugin)
{
}

GAIM_INIT_PLUGIN(e2new, init_plugin, info)
